@@ -54,14 +54,14 @@ static CamelMessageInfo *message_info_load(CamelFolderSummary *s, FILE *in);
 static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary *, struct _camel_header_raw *);
 static void message_info_free(CamelFolderSummary *, CamelMessageInfo *mi);
 
-static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelException *ex);
-static int maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex);
-static int maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex);
+static gint maildir_summary_load(CamelLocalSummary *cls, gint forceindex, CamelException *ex);
+static gint maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex);
+static gint maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex);
 static CamelMessageInfo *maildir_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *, CamelException *ex);
 
-static char *maildir_summary_next_uid_string(CamelFolderSummary *s);
-static int maildir_summary_decode_x_evolution(CamelLocalSummary *cls, const char *xev, CamelLocalMessageInfo *mi);
-static char *maildir_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessageInfo *mi);
+static gchar *maildir_summary_next_uid_string(CamelFolderSummary *s);
+static gint maildir_summary_decode_x_evolution(CamelLocalSummary *cls, const gchar *xev, CamelLocalMessageInfo *mi);
+static gchar *maildir_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessageInfo *mi);
 
 static void camel_maildir_summary_class_init	(CamelMaildirSummaryClass *class);
 static void camel_maildir_summary_init	(CamelMaildirSummary *gspaper);
@@ -70,8 +70,8 @@ static void camel_maildir_summary_finalise	(CamelObject *obj);
 #define _PRIVATE(x) (((CamelMaildirSummary *)(x))->priv)
 
 struct _CamelMaildirSummaryPrivate {
-	char *current_file;
-	char *hostname;
+	gchar *current_file;
+	gchar *hostname;
 
 	GHashTable *load_map;
 	GMutex *summary_lock;
@@ -83,7 +83,7 @@ CamelType
 camel_maildir_summary_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register(camel_local_summary_get_type (), "CamelMaildirSummary",
 					   sizeof(CamelMaildirSummary),
@@ -93,7 +93,7 @@ camel_maildir_summary_get_type (void)
 					   (CamelObjectInitFunc)camel_maildir_summary_init,
 					   (CamelObjectFinalizeFunc)camel_maildir_summary_finalise);
 	}
-	
+
 	return type;
 }
 
@@ -123,7 +123,7 @@ static void
 camel_maildir_summary_init (CamelMaildirSummary *o)
 {
 	struct _CamelFolderSummary *s = (CamelFolderSummary *) o;
-	char hostname[256];
+	gchar hostname[256];
 
 	o->priv = g_malloc0(sizeof(*o->priv));
 	/* set unique file version */
@@ -132,10 +132,6 @@ camel_maildir_summary_init (CamelMaildirSummary *o)
 	s->message_info_size = sizeof(CamelMaildirMessageInfo);
 	s->content_info_size = sizeof(CamelMaildirMessageContentInfo);
 
-#if defined (DOEPOOLV) || defined (DOESTRV)
-	s->message_info_strings = CAMEL_MAILDIR_INFO_LAST;
-#endif
-
 	if (gethostname(hostname, 256) == 0) {
 		o->priv->hostname = g_strdup(hostname);
 	} else {
@@ -161,11 +157,11 @@ camel_maildir_summary_finalise(CamelObject *obj)
  * @index: Index if one is reqiured.
  *
  * Create a new CamelMaildirSummary object.
- * 
+ *
  * Return value: A new #CamelMaildirSummary object.
  **/
 CamelMaildirSummary
-*camel_maildir_summary_new(struct _CamelFolder *folder, const char *filename, const char *maildirdir, CamelIndex *index)
+*camel_maildir_summary_new(struct _CamelFolder *folder, const gchar *filename, const gchar *maildirdir, CamelIndex *index)
 {
 	CamelMaildirSummary *o = (CamelMaildirSummary *)camel_object_new(camel_maildir_summary_get_type ());
 
@@ -181,7 +177,7 @@ CamelMaildirSummary
 
 /* the 'standard' maildir flags.  should be defined in sorted order. */
 static struct {
-	char flag;
+	gchar flag;
 	guint32 flagbit;
 } flagbits[] = {
 	{ 'D', CAMEL_MESSAGE_DRAFT },
@@ -193,11 +189,11 @@ static struct {
 };
 
 /* convert the uid + flags into a unique:info maildir format */
-char *camel_maildir_summary_info_to_name(const CamelMaildirMessageInfo *info)
+gchar *camel_maildir_summary_info_to_name(const CamelMaildirMessageInfo *info)
 {
-	const char *uid;
-	char *p, *buf;
-	int i;
+	const gchar *uid;
+	gchar *p, *buf;
+	gint i;
 
 	uid = camel_message_info_uid (info);
 	buf = g_alloca (strlen (uid) + strlen (":2,") +  (sizeof (flagbits) / sizeof (flagbits[0])) + 1);
@@ -208,17 +204,17 @@ char *camel_maildir_summary_info_to_name(const CamelMaildirMessageInfo *info)
 	}
 
 	*p = 0;
-	
+
 	return g_strdup(buf);
 }
 
 /* returns 0 if the info matches (or there was none), otherwise we changed it */
-int camel_maildir_summary_name_to_info(CamelMaildirMessageInfo *info, const char *name)
+gint camel_maildir_summary_name_to_info(CamelMaildirMessageInfo *info, const gchar *name)
 {
-	char *p, c;
+	gchar *p, c;
 	guint32 set = 0;	/* what we set */
 	/*guint32 all = 0;*/	/* all flags */
-	int i;
+	gint i;
 
 	p = strstr (name, ":2,");
 
@@ -248,12 +244,12 @@ int camel_maildir_summary_name_to_info(CamelMaildirMessageInfo *info, const char
 }
 
 /* for maildir, x-evolution isn't used, so dont try and get anything out of it */
-static int maildir_summary_decode_x_evolution(CamelLocalSummary *cls, const char *xev, CamelLocalMessageInfo *mi)
+static gint maildir_summary_decode_x_evolution(CamelLocalSummary *cls, const gchar *xev, CamelLocalMessageInfo *mi)
 {
 	return -1;
 }
 
-static char *maildir_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessageInfo *mi)
+static gchar *maildir_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessageInfo *mi)
 {
 	return NULL;
 }
@@ -282,7 +278,7 @@ static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary * s, st
 	CamelMessageInfo *mi, *info;
 	CamelMaildirSummary *mds = (CamelMaildirSummary *)s;
 	CamelMaildirMessageInfo *mdi;
-	const char *uid;
+	const gchar *uid;
 
 	mi = ((CamelFolderSummaryClass *) parent_class)->message_info_new_from_header(s, h);
 	/* assign the uid and new filename */
@@ -294,7 +290,7 @@ static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary * s, st
 			mdi->info.info.uid = camel_pstring_add (camel_folder_summary_next_uid_string(s), TRUE);
 
 		/* handle 'duplicates' */
-		info = camel_folder_summary_uid(s, uid);
+		info = camel_folder_summary_peek_info (s, uid);
 		if (info) {
 			d(printf("already seen uid '%s', just summarising instead\n", uid));
 			camel_message_info_free(mi);
@@ -306,8 +302,8 @@ static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary * s, st
 
 		if (mds->priv->current_file) {
 #if 0
-			char *p1, *p2, *p3;
-			unsigned long uid;
+			gchar *p1, *p2, *p3;
+			gulong uid;
 #endif
 			/* if setting from a file, grab the flags from it */
 			camel_maildir_info_set_filename(mi, g_strdup(mds->priv->current_file));
@@ -339,20 +335,17 @@ static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary * s, st
 	return mi;
 }
 
-
 static void
 message_info_free(CamelFolderSummary *s, CamelMessageInfo *mi)
 {
-#if !defined (DOEPOOLV) && !defined (DOESTRV)
 	CamelMaildirMessageInfo *mdi = (CamelMaildirMessageInfo *)mi;
 
 	g_free(mdi->filename);
-#endif
+
 	((CamelFolderSummaryClass *) parent_class)->message_info_free(s, mi);
 }
 
-
-static char *maildir_summary_next_uid_string(CamelFolderSummary *s)
+static gchar *maildir_summary_next_uid_string(CamelFolderSummary *s)
 {
 	CamelMaildirSummary *mds = (CamelMaildirSummary *)s;
 
@@ -360,7 +353,7 @@ static char *maildir_summary_next_uid_string(CamelFolderSummary *s)
 
 	/* if we have a current file, then use that to get the uid */
 	if (mds->priv->current_file) {
-		char *cln;
+		gchar *cln;
 
 		cln = strchr(mds->priv->current_file, ':');
 		if (cln)
@@ -373,9 +366,9 @@ static char *maildir_summary_next_uid_string(CamelFolderSummary *s)
 		return g_strdup_printf("%ld.%d_%u.%s", time(0), getpid(), camel_folder_summary_next_uid(s), mds->priv->hostname);
 #else
 		CamelLocalSummary *cls = (CamelLocalSummary *)s;
-		char *name = NULL, *uid = NULL;
+		gchar *name = NULL, *uid = NULL;
 		struct stat st;
-		int retry = 0;
+		gint retry = 0;
 		guint32 nextuid = camel_folder_summary_next_uid(s);
 
 		/* we use time.pid_count.hostname */
@@ -406,7 +399,7 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 
 	mi = ((CamelFolderSummaryClass *) parent_class)->message_info_load(s, in);
 	if (mi) {
-		char *name;
+		gchar *name;
 
 		if (mds->priv->load_map
 		    && (name = g_hash_table_lookup(mds->priv->load_map, camel_message_info_uid(mi)))) {
@@ -419,15 +412,15 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 	return mi;
 }
 
-static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelException *ex)
+static gint maildir_summary_load(CamelLocalSummary *cls, gint forceindex, CamelException *ex)
 {
-	char *cur;
+	gchar *cur;
 	DIR *dir;
 	struct dirent *d;
 	CamelMaildirSummary *mds = (CamelMaildirSummary *)cls;
-	char *uid;
+	gchar *uid;
 	EMemPool *pool;
-	int ret;
+	gint ret;
 
 	cur = g_strdup_printf("%s/cur", cls->folder_path);
 
@@ -448,11 +441,11 @@ static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelExc
 	while ( (d = readdir(dir)) ) {
 		if (d->d_name[0] == '.')
 			continue;
-		
+
 		/* map the filename -> uid */
 		uid = strchr(d->d_name, ':');
 		if (uid) {
-			int len = uid-d->d_name;
+			gint len = uid-d->d_name;
 			uid = e_mempool_alloc(pool, len+1);
 			memcpy(uid, d->d_name, len);
 			uid[len] = 0;
@@ -474,19 +467,19 @@ static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelExc
 	return ret;
 }
 
-static int
-camel_maildir_summary_add (CamelLocalSummary *cls, const char *name, int forceindex)
+static gint
+camel_maildir_summary_add (CamelLocalSummary *cls, const gchar *name, gint forceindex)
 {
 	CamelMaildirSummary *maildirs = (CamelMaildirSummary *)cls;
-	char *filename = g_strdup_printf("%s/cur/%s", cls->folder_path, name);
-	int fd;
+	gchar *filename = g_strdup_printf("%s/cur/%s", cls->folder_path, name);
+	gint fd;
 	CamelMimeParser *mp;
 
 	d(printf("summarising: %s\n", name));
 
 	fd = open(filename, O_RDONLY|O_LARGEFILE);
 	if (fd == -1) {
-		g_warning ("Cannot summarise/index: %s: %s", filename, strerror (errno));
+		g_warning ("Cannot summarise/index: %s: %s", filename, g_strerror (errno));
 		g_free(filename);
 		return -1;
 	}
@@ -499,7 +492,7 @@ camel_maildir_summary_add (CamelLocalSummary *cls, const char *name, int forcein
 	} else {
 		camel_folder_summary_set_index((CamelFolderSummary *)maildirs, NULL);
 	}
-	maildirs->priv->current_file = (char *)name;
+	maildirs->priv->current_file = (gchar *)name;
 	camel_folder_summary_add_from_parser((CamelFolderSummary *)maildirs, mp);
 	camel_object_unref((CamelObject *)mp);
 	maildirs->priv->current_file = NULL;
@@ -514,7 +507,7 @@ struct _remove_data {
 };
 
 static void
-remove_summary(char *key, CamelMessageInfo *info, struct _remove_data *rd)
+remove_summary(gchar *key, CamelMessageInfo *info, struct _remove_data *rd)
 {
 	d(printf("removing message %s from summary\n", key));
 	if (rd->cls->index)
@@ -525,20 +518,20 @@ remove_summary(char *key, CamelMessageInfo *info, struct _remove_data *rd)
 	camel_message_info_free(info);
 }
 
-static int
+static gint
 maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, CamelException *ex)
 {
 	DIR *dir;
 	struct dirent *d;
-	char *p;
+	gchar *p;
 	CamelMessageInfo *info;
 	CamelMaildirMessageInfo *mdi;
 	CamelFolderSummary *s = (CamelFolderSummary *)cls;
 	GHashTable *left;
-	int i, count, total;
-	int forceindex;
-	char *new, *cur;
-	char *uid;
+	gint i, count, total;
+	gint forceindex;
+	gchar *new, *cur;
+	gchar *uid;
 	struct _remove_data rd = { cls, changes };
 
 	g_mutex_lock (((CamelMaildirSummary *) cls)->priv->summary_lock);
@@ -566,12 +559,16 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 
 	/* keeps track of all uid's that have not been processed */
 	left = g_hash_table_new(g_str_hash, g_str_equal);
-	count = camel_folder_summary_count((CamelFolderSummary *)cls);
+	count = camel_folder_summary_count (s);
+	if (count != camel_folder_summary_cache_size (s)) {
+		camel_folder_summary_reload_from_db (s, ex);
+		count = camel_folder_summary_count (s);
+	}
 	forceindex = count == 0;
 	for (i=0;i<count;i++) {
 		info = camel_folder_summary_index((CamelFolderSummary *)cls, i);
 		if (info) {
-			g_hash_table_insert(left, (char *)camel_message_info_uid(info), info);
+			g_hash_table_insert(left, (gchar *)camel_message_info_uid(info), info);
 		}
 	}
 
@@ -583,7 +580,7 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 	rewinddir(dir);
 
 	while ( (d = readdir(dir)) ) {
-		int pc = count * 100 / total;
+		gint pc = count * 100 / total;
 
 		camel_operation_progress(NULL, pc);
 		count++;
@@ -599,7 +596,7 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 			uid = g_strndup(d->d_name, uid-d->d_name);
 		else
 			uid = g_strdup(d->d_name);
-		
+
 		info = g_hash_table_lookup(left, uid);
 		if (info) {
 			camel_message_info_free(info);
@@ -613,7 +610,7 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 				if (changes)
 					camel_folder_change_info_add_uid(changes, uid);
 		} else {
-			const char *filename;
+			const gchar *filename;
 
 			if (cls->index && (!camel_index_has_name(cls->index, uid))) {
 				/* message_info_new will handle duplicates */
@@ -624,23 +621,8 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 			filename = camel_maildir_info_filename(mdi);
 			/* TODO: only store the extension in the mdi->filename struct, not the whole lot */
 			if (filename == NULL || strcmp(filename, d->d_name) != 0) {
-#ifdef DOESTRV
-#warning "cannot modify the estrv after its been setup, for mt-safe code"
-				CAMEL_SUMMARY_LOCK(s, summary_lock);
-				/* need to update the summary hash ref */
-				g_hash_table_remove(s->messages_uid, camel_message_info_uid(info));
-				info->strings = e_strv_set_ref(info->strings, CAMEL_MAILDIR_INFO_FILENAME, d->d_name);
-				info->strings = e_strv_pack(info->strings);
-				g_hash_table_insert(s->messages_uid, (char *)camel_message_info_uid(info), info);
-				CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-#else
-# ifdef DOEPOOLV
-				info->strings = e_poolv_set(info->strings, CAMEL_MAILDIR_INFO_FILENAME, d->d_name, FALSE);
-# else	
 				g_free(mdi->filename);
 				mdi->filename = g_strdup(d->d_name);
-# endif
-#endif
 			}
 			camel_message_info_free(info);
 		}
@@ -664,9 +646,9 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 		rewinddir(dir);
 
 		while ( (d = readdir(dir)) ) {
-			char *name, *newname, *destname, *destfilename;
-			char *src, *dest;
-			int pc = count * 100 / total;
+			gchar *name, *newname, *destname, *destfilename;
+			gchar *src, *dest;
+			gint pc = count * 100 / total;
 
 			camel_operation_progress(NULL, pc);
 			count++;
@@ -680,8 +662,12 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 				camel_message_info_free(info);
 				newname = destname = camel_folder_summary_next_uid_string(s);
 			} else {
-				newname = NULL;
-				destname = name;
+				gchar *nm;
+				newname = g_strdup (name);
+				nm =strrchr (newname, ':');
+				if (nm)
+					*nm = '\0';
+				destname = newname;
 			}
 
 			/* copy this to the destination folder, use 'standard' semantics for maildir info field */
@@ -709,8 +695,8 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 			g_free(dest);
 		}
 		camel_operation_end(NULL);
+		closedir(dir);
 	}
-	closedir(dir);
 
 	g_free(new);
 	g_free(cur);
@@ -721,16 +707,13 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 }
 
 /* sync the summary with the ondisk files. */
-static int
+static gint
 maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changes, CamelException *ex)
 {
-	int count, i;
+	gint count, i;
 	CamelMessageInfo *info;
 	CamelMaildirMessageInfo *mdi;
-#ifdef DOESTRV
-	CamelFolderSummary *s = (CamelFolderSummary *)cls;
-#endif
-	char *name;
+	gchar *name;
 	struct stat st;
 
 	d(printf("summary_sync(expunge=%s)\n", expunge?"true":"false"));
@@ -760,8 +743,8 @@ maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChange
 			}
 			g_free(name);
 		} else if (mdi && (mdi->info.info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
-			char *newname = camel_maildir_summary_info_to_name(mdi);
-			char *dest;
+			gchar *newname = camel_maildir_summary_info_to_name(mdi);
+			gchar *dest;
 
 			/* do we care about additional metainfo stored inside the message? */
 			/* probably should all go in the filename? */
@@ -778,23 +761,8 @@ maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChange
 					/* TODO: If this is made mt-safe, then this code could be a problem, since
 					   the estrv is being modified.
 					   Sigh, this may mean the maildir name has to be cached another way */
-#ifdef DOESTRV
-#warning "cannot modify the estrv after its been setup, for mt-safe code"
-					CAMEL_SUMMARY_LOCK(s, summary_lock);
-					/* need to update the summary hash ref */
-					g_hash_table_remove(s->messages_uid, camel_message_info_uid(info));
-					info->strings = e_strv_set_ref_free(info->strings, CAMEL_MAILDIR_INFO_FILENAME, newname);
-					info->strings = e_strv_pack(info->strings);
-					g_hash_table_insert(s->messages_uid, (char *)camel_message_info_uid(info), info);
-					CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-#else
-# ifdef DOEPOOLV
-					info->strings = e_poolv_set(info->strings, CAMEL_MAILDIR_INFO_FILENAME, newname, TRUE);
-# else
 					g_free(mdi->filename);
 					mdi->filename = newname;
-# endif
-#endif
 				}
 				g_free(name);
 				g_free(dest);
